@@ -37,12 +37,11 @@ Launcher::~Launcher() {
 
 }
 
-bool Launcher::openFolder(GAppLaunchContext* ctx, GList* folder_infos, GError** err) {
-    GList* l = folder_infos;
-    FmFileInfo* fi = FM_FILE_INFO(l->data);
+bool Launcher::openFolder(GAppLaunchContext* /*ctx*/, const Fm::FileInfoList& folderInfos, Fm::GErrorPtr& /*err*/) {
+    auto fi = folderInfos[0];
     Application* app = static_cast<Application*>(qApp);
     MainWindow* mainWindow = mainWindow_;
-    Fm::FilePath path{fm_path_to_gfile(fm_file_info_get_path(fi)), false};
+    Fm::FilePath path = fi->path();
     if(!mainWindow) {
         mainWindow = new MainWindow(std::move(path));
         mainWindow->resize(app->settings().windowWidth(), app->settings().windowHeight());
@@ -54,10 +53,10 @@ bool Launcher::openFolder(GAppLaunchContext* ctx, GList* folder_infos, GError**
     else {
         mainWindow->chdir(std::move(path));
     }
-    l = l->next;
-    for(; l; l = l->next) {
-        fi = FM_FILE_INFO(l->data);
-        path = Fm::FilePath{fm_path_to_gfile(fm_file_info_get_path(fi)), false};
+
+    for(size_t i = 1; i < folderInfos.size(); ++i) {
+        fi = folderInfos[i];
+        path = fi->path();
         mainWindow->addTab(std::move(path));
     }
     mainWindow->show();
