@@ -26,12 +26,12 @@
 #include <QSortFilterProxyModel>
 #include <QLineEdit>
 #include <QTabWidget>
-#include <libfm/fm.h>
 #include <QMessageBox>
 #include <QTabBar>
 #include <QStackedWidget>
 #include <QSplitter>
 #include "launcher.h"
+#include "tabbar.h"
 #include <libfm-qt/core/filepath.h>
 #include <libfm-qt/core/bookmarks.h>
 
@@ -42,6 +42,33 @@ class PathBar;
 
 namespace PCManFM {
 
+class ViewFrame : public QFrame {
+    Q_OBJECT
+public:
+    ViewFrame(QWidget* parent = nullptr);
+    ~ViewFrame() {};
+
+    void createTopBar(bool usePathButtons);
+    void removeTopBar();
+
+    QWidget* getTopBar() const {
+        return topBar_;
+    }
+    TabBar* getTabBar() const {
+        return tabBar_;
+    }
+    QStackedWidget* getStackedWidget() const {
+        return stackedWidget_;
+    }
+
+private:
+    QWidget* topBar_;
+    TabBar* tabBar_;
+    QStackedWidget* stackedWidget_;
+};
+
+//======================================================================
+
 class TabPage;
 class Settings;
 
@@ -51,11 +78,21 @@ public:
     MainWindow(Fm::FilePath path = Fm::FilePath());
     virtual ~MainWindow();
 
-    void chdir(Fm::FilePath path);
-    int addTab(Fm::FilePath path);
+    void chdir(Fm::FilePath path, ViewFrame* viewFrame);
+    void chdir(Fm::FilePath path) {
+        chdir(path, activeViewFrame_);
+    }
 
+    int addTab(Fm::FilePath path, ViewFrame* viewFrame);
+    int addTab(Fm::FilePath path) {
+        return addTab(path, activeViewFrame_);
+    }
+
+    TabPage* currentPage(ViewFrame* viewFrame) {
+        return reinterpret_cast<TabPage*>(viewFrame->getStackedWidget()->currentWidget());
+    }
     TabPage* currentPage() {
-        return reinterpret_cast<TabPage*>(ui.stackedWidget->currentWidget());
+        return currentPage(activeViewFrame_);
     }
 
     void updateFromSettings(Settings& settings);
@@ -103,11 +140,13 @@ protected Q_SLOTS:
 
     void on_actionGo_triggered();
     void on_actionShowHidden_triggered(bool check);
+    void on_actionSplitView_triggered(bool check);
     void on_actionPreserveView_triggered(bool checked);
 
     void on_actionByFileName_triggered(bool checked);
     void on_actionByMTime_triggered(bool checked);
     void on_actionByOwner_triggered(bool checked);
+    void on_actionByGroup_triggered(bool checked);
     void on_actionByFileType_triggered(bool checked);
     void on_actionByFileSize_triggered(bool checked);
     void on_actionAscending_triggered(bool checked);
@@ -115,6 +154,8 @@ protected Q_SLOTS:
     void on_actionFolderFirst_triggered(bool checked);
     void on_actionCaseSensitive_triggered(bool checked);
     void on_actionFilter_triggered(bool checked);
+    void on_actionUnfilter_triggered();
+    void on_actionShowFilter_triggered();
 
     void on_actionLocationBar_triggered(bool checked);
     void on_actionPathButtons_triggered(bool checked);
@@ -129,6 +170,7 @@ protected Q_SLOTS:
 
     void on_actionOpenTerminal_triggered();
     void on_actionOpenAsRoot_triggered();
+    void on_actionCopyFullPath_triggered();
     void on_actionFindFiles_triggered();
 
     void on_actionAbout_triggered();
@@ -139,9 +181,6 @@ protected Q_SLOTS:
     void onTabBarCurrentChanged(int index);
     void onTabBarTabMoved(int from, int to);
 
-    void focusFilterBar();
-    void onFilterStringChanged(QString str);
-
     void onShortcutPrevTab();
     void onShortcutNextTab();
     void onShortcutJumpToTab();
@@ -164,7 +203,10 @@ protected Q_SLOTS:
 
     void onBackForwardContextMenu(QPoint pos);
 
+    void onFolderUnmounted();
+
     void tabContextMenu(const QPoint& pos);
+    void onTabBarClicked(int index);
     void closeLeftTabs();
     void closeRightTabs();
     void closeOtherTabs() {
@@ -182,21 +224,27 @@ protected Q_SLOTS:
 protected:
     bool event(QEvent* event) override;
     void changeEvent(QEvent* event) override;
-    void closeTab(int index);
+    void closeTab(int index, ViewFrame* viewFrame);
+    void closeTab(int index) {
+        closeTab(index, activeViewFrame_);
+    }
     virtual void resizeEvent(QResizeEvent* event) override;
     virtual void closeEvent(QCloseEvent* event) override;
     virtual void dragEnterEvent(QDragEnterEvent* event) override;
     virtual void dropEvent(QDropEvent* event) override;
+    virtual bool eventFilter(QObject* watched, QEvent* event);
 
 private:
     void loadBookmarksMenu();
-    void updateUIForCurrentPage();
+    void updateUIForCurrentPage(bool setFocus = true);
     void updateViewMenuForCurrentPage();
     void updateEditSelectedActions();
     void updateStatusBarForCurrentPage();
     void setRTLIcons(bool isRTL);
     void createPathBar(bool usePathButtons);
-    int addTabWithPage(TabPage* page, Fm::FilePath path = Fm::FilePath());
+    void addViewFrame(const Fm::FilePath& path);
+    ViewFrame* viewFrameForTabPage(TabPage* page);
+    int addTabWithPage(TabPage* page, ViewFrame* viewFrame, Fm::FilePath path = Fm::FilePath());
     void dropTab();
 
 private:
@@ -209,6 +257,12 @@ private:
     int rightClickIndex_;
     bool updatingViewMenu_;
     QAction* menuSep_;
+    QAction* menuSpacer_;
+
+    ViewFrame* activeViewFrame_;
+    // The split mode of this window is changed only from its settings,
+    // not from another window. So, we get the mode at the start and keep it.
+    bool splitView_;
 
     static MainWindow* lastActive_;
 };
