@@ -22,13 +22,13 @@
 #define PCMANFM_SETTINGS_H
 
 #include <QObject>
-#include <libfm/fm.h>
 #include <libfm-qt/folderview.h>
 #include <libfm-qt/foldermodel.h>
 #include "desktopwindow.h"
 #include <libfm-qt/sidepane.h>
 #include <libfm-qt/core/thumbnailjob.h>
 #include <libfm-qt/core/archiver.h>
+#include <libfm-qt/core/legacy/fm-config.h>
 
 namespace PCManFM {
 
@@ -313,6 +313,14 @@ public:
         desktopIconSize_ = desktopIconSize;
     }
 
+    QStringList desktopShortcuts() const {
+        return desktopShortcuts_;
+    }
+
+    void setDesktopShortcuts(const QStringList& list) {
+        desktopShortcuts_ = list;
+    }
+
     bool showWmMenu() const {
         return showWmMenu_;
     }
@@ -464,12 +472,12 @@ public:
         showMenuBar_ = showMenuBar;
     }
 
-    bool fullWidthTabBar() const {
-        return fullWidthTabBar_;
+    bool splitView() const {
+        return splitView_;
     }
 
-    void setFullWidthTabBar(bool fullWith) {
-        fullWidthTabBar_ = fullWith;
+    void setSplitView(bool split) {
+        splitView_ = split;
     }
 
     Fm::FolderView::ViewMode viewMode() const {
@@ -870,6 +878,36 @@ public:
         searchhHidden_ = hidden;
     }
 
+    QList<int> getCustomColumnWidths() const {
+        QList<int> l;
+        for(auto width : qAsConst(customColumnWidths_)) {
+            l << width.toInt();
+        }
+        return l;
+    }
+
+    void setCustomColumnWidths(const QList<int> &widths) {
+        customColumnWidths_.clear();
+        for(auto width : widths) {
+            customColumnWidths_ << QVariant(width);
+        }
+    }
+
+    QList<int> getHiddenColumns() const {
+        QList<int> l;
+        for(auto width : qAsConst(hiddenColumns_)) {
+            l << width.toInt();
+        }
+        return l;
+    }
+
+    void setHiddenColumns(const QList<int> &columns) {
+        hiddenColumns_.clear();
+        for(auto column : columns) {
+            hiddenColumns_ << QVariant(column);
+        }
+    }
+
 private:
     int toIconSize(int size, IconType type) const;
 
@@ -899,6 +937,7 @@ private:
     QColor desktopShadowColor_;
     QFont desktopFont_;
     int desktopIconSize_;
+    QStringList desktopShortcuts_;
     bool showWmMenu_;
 
     bool desktopShowHidden_;
@@ -918,7 +957,7 @@ private:
     int splitterPos_;
     Fm::SidePane::Mode sidePaneMode_;
     bool showMenuBar_;
-    bool fullWidthTabBar_;
+    bool splitView_;
 
     Fm::FolderView::ViewMode viewMode_;
     bool showHidden_;
@@ -975,6 +1014,10 @@ private:
     bool searchContentRegexp_;
     bool searchRecursive_;
     bool searchhHidden_;
+
+    // detailed list columns
+    QList<QVariant> customColumnWidths_;
+    QList<QVariant> hiddenColumns_;
 };
 
 }
