@@ -30,6 +30,8 @@
 #include <QCursor>
 #include <QMessageBox>
 #include <QScrollBar>
+#include <QToolButton>
+#include <QLabel>
 #include <QDir>
 #include "settings.h"
 #include "application.h"
@@ -52,6 +54,50 @@ bool ProxyFilter::filterAcceptsRow(const Fm::ProxyFolderModel* model, const std:
     return true;
 }
 
+//==================================================
+
+FilterEdit::FilterEdit(QWidget* parent) : QLineEdit(parent) {
+    setClearButtonEnabled(true);
+    if(QToolButton *clearButton = findChild<QToolButton*>()) {
+        clearButton->setToolTip(tr("Clear text (Ctrl+K)"));
+    }
+}
+
+void FilterEdit::keyPressEvent(QKeyEvent* event) {
+    // since two views can be shown in the split mode, Ctrl+K can't be
+    // used as a QShortcut but can come here for clearing the text
+    if(event->modifiers() == Qt::ControlModifier && event->key() == Qt::Key_K) {
+        clear();
+    }
+    QLineEdit::keyPressEvent(event);
+}
+
+void FilterEdit::keyPressed(QKeyEvent* event) {
+    // NOTE: Movement and delete keys should be left to the view.
+    // Copy/paste shortcuts are taken by the view but they aren't needed here
+    // (Shift+Insert works for pasting but, since most users may not be familiar
+    // with it, an action is added to the main window for focusing an empty bar).
+    if(!hasFocus()
+       && event->key() != Qt::Key_Left && event->key() != Qt::Key_Right
+       && event->key() != Qt::Key_Home && event->key() != Qt::Key_End
+       && event->key() != Qt::Key_Delete) {
+        keyPressEvent(event);
+    }
+}
+
+FilterBar::FilterBar(QWidget* parent) : QWidget(parent) {
+    QHBoxLayout* HLayout = new QHBoxLayout(this);
+    HLayout->setSpacing(5);
+    filterEdit_ = new FilterEdit();
+    QLabel *label = new QLabel(tr("Filter:"));
+    HLayout->addWidget(label);
+    HLayout->addWidget(filterEdit_);
+    connect(filterEdit_, &QLineEdit::textChanged, this, &FilterBar::textChanged);
+    connect(filterEdit_, &FilterEdit::lostFocus, this, &FilterBar::lostFocus);
+}
+
+//==================================================
+
 TabPage::TabPage(QWidget* parent):
     QWidget(parent),
     folderView_{nullptr},
@@ -60,7 +106,8 @@ TabPage::TabPage(QWidget* parent):
     proxyFilter_{nullptr},
     verticalLayout{nullptr},
     overrideCursor_(false),
-    selectionTimer_(nullptr) {
+    selectionTimer_(nullptr),
+    filterBar_(nullptr) {
 
     Settings& settings = static_cast<Application*>(qApp)->settings();
 
@@ -69,25 +116,47 @@ TabPage::TabPage(QWidget* parent):
     proxyModel_->setShowHidden(settings.showHidden());
     proxyModel_->setBackupAsHidden(settings.backupAsHidden());
     proxyModel_->setShowThumbnails(settings.showThumbnails());
-    connect(proxyModel_, &ProxyFolderModel::sortFilterChanged, this, &TabPage::sortFilterChanged);
+    connect(proxyModel_, &ProxyFolderModel::sortFilterChanged, this, [this] {
+        saveFolderSorting();
+        Q_EMIT sortFilterChanged();
+    });
 
     verticalLayout = new QVBoxLayout(this);
     verticalLayout->setContentsMargins(0, 0, 0, 0);
 
     folderView_ = new View(settings.viewMode(), this);
     folderView_->setMargins(settings.folderViewCellMargins());
+    folderView_->setShadowHidden(settings.shadowHidden());
     // newView->setColumnWidth(Fm::FolderModel::ColumnName, 200);
     connect(folderView_, &View::openDirRequested, this, &TabPage::openDirRequested);
     connect(folderView_, &View::selChanged, this, &TabPage::onSelChanged);
     connect(folderView_, &View::clickedBack, this, &TabPage::backwardRequested);
     connect(folderView_, &View::clickedForward, this, &TabPage::forwardRequested);
 
+    // customization of columns of detailed list view
+    folderView_->setCustomColumnWidths(settings.getCustomColumnWidths());
+    folderView_->setHiddenColumns(settings.getHiddenColumns());
+    connect(folderView_, &View::columnResizedByUser, this, [this, &settings]() {
+        settings.setCustomColumnWidths(folderView_->getCustomColumnWidths());
+    });
+    connect(folderView_, &View::columnHiddenByUser, this, [this, &settings]() {
+        settings.setHiddenColumns(folderView_->getHiddenColumns());
+    });
+
     proxyFilter_ = new ProxyFilter();
     proxyModel_->addFilter(proxyFilter_);
 
     // FIXME: this is very dirty
     folderView_->setModel(proxyModel_);
     verticalLayout->addWidget(folderView_);
+
+    // filter-bar and its settings
+    filterBar_ = new FilterBar();
+    verticalLayout->addWidget(filterBar_);
+    if(!settings.showFilter()){
+        transientFilterBar(true);
+    }
+    connect(filterBar_, &FilterBar::textChanged, this, &TabPage::onFilterStringChanged);
 }
 
 TabPage::~TabPage() {
@@ -109,6 +178,78 @@ TabPage::~TabPage() {
     }
 }
 
+void TabPage::transientFilterBar(bool transient) {
+    if(filterBar_) {
+        filterBar_->clear();
+        if(transient) {
+            filterBar_->hide();
+            folderView_->childView()->removeEventFilter(this);
+            folderView_->childView()->installEventFilter(this);
+            connect(filterBar_, &FilterBar::lostFocus, this, &TabPage::onLosingFilterBarFocus);
+        }
+        else {
+            filterBar_->show();
+            folderView_->childView()->removeEventFilter(this);
+            disconnect(filterBar_, &FilterBar::lostFocus, this, &TabPage::onLosingFilterBarFocus);
+        }
+    }
+}
+
+void TabPage::onLosingFilterBarFocus() {
+    // hide the empty transient filter-bar when it loses focus
+    if(getFilterStr().isEmpty()) {
+        filterBar_->hide();
+    }
+}
+
+void TabPage::showFilterBar() {
+    if(filterBar_) {
+        filterBar_->show();
+        if(isVisibleTo(this)) { // the page itself may be in an inactive tab
+            filterBar_->focusBar();
+        }
+    }
+}
+
+bool TabPage::eventFilter(QObject* watched, QEvent* event) {
+    // when a text is typed inside the view, type it inside the filter-bar
+    if(filterBar_ && watched == folderView_->childView() &&  event->type() == QEvent::KeyPress) {
+        if(QKeyEvent* ke = static_cast<QKeyEvent*>(event)) {
+            filterBar_->keyPressed(ke);
+        }
+    }
+    return QWidget::eventFilter(watched, event);
+}
+
+void TabPage::backspacePressed() {
+    if(filterBar_ && filterBar_->isVisible()) {
+        QKeyEvent bs = QKeyEvent(QEvent::KeyPress, Qt::Key_Backspace, Qt::NoModifier);
+        filterBar_->keyPressed(&bs);
+    }
+}
+
+void TabPage::onFilterStringChanged(QString str) {
+    if(filterBar_ && str != getFilterStr()) {
+        setFilterStr(str);
+        applyFilter();
+        // show/hide the transient filter-bar appropriately
+        if(!static_cast<Application*>(qApp)->settings().showFilter()) {
+            if(filterBar_->isVisibleTo(this)) { // the page itself may be in an inactive tab
+                if(str.isEmpty()) {
+                    // focus the view BEFORE hiding the filter-bar to avoid redundant "FocusIn" events;
+                    // otherwise, another widget inside the main window might gain focus immediately
+                    // after the filter-bar is hidden and only after that, the view will be focused.
+                    folderView()->childView()->setFocus();
+                    filterBar_->hide();
+                }
+            }
+            else if(!str.isEmpty()) {
+                filterBar_->show();
+            }
+        }
+    }
+}
+
 void TabPage::freeFolder() {
     if(folder_) {
         if(folderSettings_.isCustomized()) {
@@ -285,13 +426,9 @@ void TabPage::onFolderFsInfo() {
     guint64 free, total;
     QString& msg = statusText_[StatusTextFSInfo];
     if(folder_->getFilesystemInfo(&total, &free)) {
-        char total_str[64];
-        char free_str[64];
-        fm_file_size_to_str(free_str, sizeof(free_str), free, fm_config->si_unit);
-        fm_file_size_to_str(total_str, sizeof(total_str), total, fm_config->si_unit);
         msg = tr("Free space: %1 (Total: %2)")
-              .arg(QString::fromUtf8(free_str),
-              QString::fromUtf8(total_str));
+              .arg(formatFileSize(free, fm_config->si_unit))
+              .arg(formatFileSize(total, fm_config->si_unit));
     }
     else {
         msg.clear();
@@ -340,21 +477,12 @@ void TabPage::onFolderRemoved() {
 void TabPage::onFolderUnmount() {
     // the folder we're showing is unmounted, destroy the widget
     qDebug("folder unmount");
-    // NOTE: call deleteLater() directly from this GObject signal handler
-    // does not work but I don't know why.
-    // Maybe it's the problem of glib mainloop integration?
-    // Call it when idle works, though.
-    Settings& settings = static_cast<Application*>(qApp)->settings();
-    // NOTE: call deleteLater() directly from this GObject signal handler
-    // does not work but I don't know why.
-    // Maybe it's the problem of glib mainloop integration?
-    // Call it when idle works, though.
-    if(settings.closeOnUnmount()) {
-        QTimer::singleShot(0, this, SLOT(deleteLater()));
-    }
-    else {
-        chdir(Fm::FilePath::homeDir());
-    }
+    // NOTE: We cannot delete the page or change its directory here
+    // because unmounting might be done from places view, in which case,
+    // the mount operation is a child of the places view and should be
+    // finished before doing anything else.
+    freeFolder();
+    Q_EMIT folderUnmounted();
 }
 
 void TabPage::onFolderContentChanged() {
@@ -372,6 +500,9 @@ QString TabPage::pathName() {
 
 void TabPage::chdir(Fm::FilePath newPath, bool addHistory) {
     // qDebug() << "TABPAGE CHDIR:" << newPath.toString().get();
+    if(filterBar_){
+        filterBar_->clear();
+    }
     if(folder_) {
         // we're already in the specified dir
         if(newPath == folder_->path()) {
@@ -421,18 +552,23 @@ void TabPage::chdir(Fm::FilePath newPath, bool addHistory) {
     connect(folder_.get(), &Fm::Folder::unmount, this, &TabPage::onFolderUnmount);
     connect(folder_.get(), &Fm::Folder::contentChanged, this, &TabPage::onFolderContentChanged);
 
-    folderModel_ = CachedFolderModel::modelFromFolder(folder_);
-
-    // set sorting, considering customized folders
     Settings& settings = static_cast<Application*>(qApp)->settings();
-    folderSettings_ = settings.loadFolderSettings(path());
-    proxyModel_->sort(folderSettings_.sortColumn(), folderSettings_.sortOrder());
-    proxyModel_->setFolderFirst(folderSettings_.sortFolderFirst());
-    proxyModel_->setShowHidden(folderSettings_.showHidden());
-    proxyModel_->setSortCaseSensitivity(folderSettings_.sortCaseSensitive() ? Qt::CaseSensitive : Qt::CaseInsensitive);
-    proxyModel_->setSourceModel(folderModel_);
+    folderModel_ = CachedFolderModel::modelFromFolder(folder_);
+    folderModel_->setShowFullName(settings.showFullNames());
 
-    folderView_->setViewMode(folderSettings_.viewMode());
+    // folderSettings_ will be set by saveFolderSorting() when the sort filter is changed below
+    // (and also by setViewMode()); here, we only need to know whether it should be saved
+    FolderSettings folderSettings = settings.loadFolderSettings(path());
+    folderSettings_.setCustomized(folderSettings.isCustomized());
+
+    // set sorting
+    proxyModel_->sort(folderSettings.sortColumn(), folderSettings.sortOrder());
+    proxyModel_->setFolderFirst(folderSettings.sortFolderFirst());
+    proxyModel_->setShowHidden(folderSettings.showHidden());
+    proxyModel_->setSortCaseSensitivity(folderSettings.sortCaseSensitive() ? Qt::CaseSensitive : Qt::CaseInsensitive);
+    proxyModel_->setSourceModel(folderModel_);
+    // set view mode
+    setViewMode(folderSettings.viewMode());
 
     if(folder_->isLoaded()) {
         onFolderStartLoading();
@@ -600,72 +736,79 @@ void TabPage::updateFromSettings(Settings& settings) {
 }
 
 void TabPage::setViewMode(Fm::FolderView::ViewMode mode) {
+    Settings& settings = static_cast<Application*>(qApp)->settings();
     if(folderSettings_.viewMode() != mode) {
         folderSettings_.setViewMode(mode);
         if(folderSettings_.isCustomized()) {
-            static_cast<Application*>(qApp)->settings().saveFolderSettings(path(), folderSettings_);
+            settings.saveFolderSettings(path(), folderSettings_);
         }
     }
+    Fm::FolderView::ViewMode prevMode = folderView_->viewMode();
     folderView_->setViewMode(mode);
+    folderView_->childView()->setFocus();
+    if(prevMode != folderView_->viewMode()) {
+        // FolderView::setViewMode() may delete the view to switch between list and tree.
+        // So, the event filter should be re-installed and the status message should be updated.
+        if(!settings.showFilter()) {
+            folderView_->childView()->removeEventFilter(this);
+            folderView_->childView()->installEventFilter(this);
+        }
+        onSelChanged();
+    }
 }
 
 void TabPage::sort(int col, Qt::SortOrder order) {
-    if(folderSettings_.sortColumn() != col || folderSettings_.sortOrder() != order) {
-        folderSettings_.setSortColumn(Fm::FolderModel::ColumnId(col));
-        folderSettings_.setSortOrder(order);
-        if(folderSettings_.isCustomized()) {
-            static_cast<Application*>(qApp)->settings().saveFolderSettings(path(), folderSettings_);
-        }
-    }
     if(proxyModel_) {
         proxyModel_->sort(col, order);
     }
 }
 
 void TabPage::setSortFolderFirst(bool value) {
-    if(folderSettings_.sortFolderFirst() != value) {
-        folderSettings_.setSortFolderFirst(value);
-        if(folderSettings_.isCustomized()) {
-            static_cast<Application*>(qApp)->settings().saveFolderSettings(path(), folderSettings_);
-        }
+    if(proxyModel_) {
+        proxyModel_->setFolderFirst(value);
     }
-    proxyModel_->setFolderFirst(value);
 }
 
 void TabPage::setSortCaseSensitive(bool value) {
-    if(folderSettings_.sortCaseSensitive() != value) {
-        folderSettings_.setSortCaseSensitive(value);
-        if(folderSettings_.isCustomized()) {
-            static_cast<Application*>(qApp)->settings().saveFolderSettings(path(), folderSettings_);
-        }
+    if(proxyModel_) {
+        proxyModel_->setSortCaseSensitivity(value ? Qt::CaseSensitive : Qt::CaseInsensitive);
     }
-    proxyModel_->setSortCaseSensitivity(value ? Qt::CaseSensitive : Qt::CaseInsensitive);
 }
 
-
 void TabPage::setShowHidden(bool showHidden) {
-    if(folderSettings_.showHidden() != showHidden) {
-        folderSettings_.setShowHidden(showHidden);
-        if(folderSettings_.isCustomized()) {
-            static_cast<Application*>(qApp)->settings().saveFolderSettings(path(), folderSettings_);
-        }
-    }
-    if(!proxyModel_) {
-        return;
-    }
-    if(showHidden != proxyModel_->showHidden()) {
+    if(proxyModel_) {
         proxyModel_->setShowHidden(showHidden);
     }
-    // this may also be called by MainWindow::onTabPageSortFilterChanged to set status message
-    statusText_[StatusTextNormal] = formatStatusText();
-    Q_EMIT statusChanged(StatusTextNormal, statusText_[StatusTextNormal]);
+}
+
+void TabPage::saveFolderSorting() {
+    if (proxyModel_ == nullptr) {
+        return;
+    }
+    folderSettings_.setSortOrder(proxyModel_->sortOrder());
+    folderSettings_.setSortColumn(static_cast<Fm::FolderModel::ColumnId>(proxyModel_->sortColumn()));
+    folderSettings_.setSortFolderFirst(proxyModel_->folderFirst());
+    folderSettings_.setSortCaseSensitive(proxyModel_->sortCaseSensitivity());
+    if(folderSettings_.showHidden() != proxyModel_->showHidden()) {
+        folderSettings_.setShowHidden(proxyModel_->showHidden());
+        statusText_[StatusTextNormal] = formatStatusText();
+        Q_EMIT statusChanged(StatusTextNormal, statusText_[StatusTextNormal]);
+    }
+    if(folderSettings_.isCustomized()) {
+        static_cast<Application*>(qApp)->settings().saveFolderSettings(path(), folderSettings_);
+    }
 }
 
 void TabPage::applyFilter() {
-    if(!proxyModel_) {
+    if(proxyModel_ == nullptr) {
         return;
     }
+    int prevSelSize = folderView_->selectionModel()->selectedIndexes().size();
     proxyModel_->updateFilters();
+    // if some selected files are filtered out, "View::selChanged()" won't be emitted
+    if(prevSelSize > folderView_->selectionModel()->selectedIndexes().size()) {
+        onSelChanged();
+    }
     statusText_[StatusTextNormal] = formatStatusText();
     Q_EMIT statusChanged(StatusTextNormal, statusText_[StatusTextNormal]);
 }
